import { useEffect, useState, useRef } from "react";
import { getAllUsers, updateUserReservedSeats } from "@/Api/Api";
import { Button } from "@/Components/Base/Buttons/Button";
import { Legend } from "@/Components/Top-level/Seat-legend/Legend";
import { SeatLayout } from "@/Components/Top-level/SeatLayout/SeatLayout";
import { getCurrentUser } from "@/lib/authService";
import styles from "@/Components/Top-level/SeatSelect/Selectseat.module.css";
import "react-toastify/dist/ReactToastify.css";
import { toast, ToastContainer } from "react-toastify";
import { Modal } from "@/Components/Base/Modal/Modal";
import { SeatSelectForm } from "@/Components/Form/SeatSelectForm/SeatSelectForm";
import { Screen } from "@/Components/Shared/Screen/Screen";
import { AiOutlineLogout } from "react-icons/ai";

export const Selectseat = ({ onLogout }) => {
  const [selectedSeats, setSelectedSeats] = useState([]);
  const [reservedSeats, setReservedSeats] = useState([]);
  const [userSeats, setUserSeats] = useState([]);
  const [seatCount, setSeatCount] = useState(1);
  const [availableSeats, setAvailableSeats] = useState(0);
  const [showSelect, setShowSelect] = useState(true);

  const hasWelcomed = useRef(false);

  useEffect(() => {
    getAllUsers()
      .then((res) => {
        const users = res.data;
        const loggedUser = getCurrentUser();
        const totalSeats = 60;

        const currentUser = users.find(
          (u) => u.phoneNumber === loggedUser.phoneNumber
        );
        const userReserved = currentUser?.reservedSeats || [];
        const allReserved = users.flatMap((u) => u.reservedSeats || []);
        const othersReserved = allReserved.filter(
          (seat) => !userReserved.includes(seat)
        );

        setUserSeats(userReserved);
        setSelectedSeats(userReserved);
        setReservedSeats(othersReserved);
        setSeatCount(userReserved.length);
        setAvailableSeats(totalSeats - allReserved.length);
        setShowSelect(true);

        if (currentUser?.name && !hasWelcomed.current) {
          toast.success(`Welcome 😊, ${currentUser.name}`);
          hasWelcomed.current = true;
        }
      })
      .catch((err) => {
        console.error("Error fetching users", err);
        toast.error("Failed to load seat data.");
      });
  }, []);

  const confirmHandler = () => {
    const user = getCurrentUser();
    if (!user || !user.phoneNumber?.trim()) {
      toast.warn("User not logged in");
      return;
    }

    const unselectedMine = userSeats.filter(
      (seat) => !selectedSeats.includes(seat)
    );
    const newlySelected = selectedSeats.filter(
      (seat) => !userSeats.includes(seat)
    );

    if (
      unselectedMine.length > 0 &&
      newlySelected.length < unselectedMine.length
    ) {
      toast.error(
        `You unselected ${unselectedMine.length} seat(s), select ${unselectedMine.length} new.`
        
      );
       hasWelcomed.current = true;
      return;
    }

    if (selectedSeats.length !== seatCount) {
      toast.error(`You must select exactly ${seatCount} seat(s).`);
       hasWelcomed.current = true;
      return;
    }

    getAllUsers()
      .then((res) => {
        const users = res.data;
        const currentUser = users.find(
          (u) => u.phoneNumber === user.phoneNumber
        );
        if (!currentUser) {
          toast.error("User not found");
          return;
        }

        const latestOthersReserved = users
          .flatMap((u) => u.reservedSeats || [])
          .filter((seat) => !currentUser.reservedSeats.includes(seat));

        const conflict = selectedSeats.filter((seat) =>
          latestOthersReserved.includes(seat)
        );
        if (conflict.length > 0) {
          toast.error(`These seats were just taken: ${conflict.join(", ")}`);
          return;
        }

        return updateUserReservedSeats(currentUser.id, selectedSeats);
      })
      .then((res) => {
        if (res) {
          toast.success("Seats booked!");
           hasWelcomed.current = true;
          localStorage.setItem("bookedSeats", JSON.stringify(selectedSeats));
          setTimeout(() => window.location.replace("/success"), 1000);
        }
      })
      .catch((err) => {
        console.error("Error confirming seats", err);
        toast.error("Failed to confirm seats.");
        setTimeout(() => window.location.replace("/error"), 1000);
      });
  };

  const handleModal = () => setShowSelect(false);

  const seatOptions = Array.from({ length: availableSeats }).map((_, i) => ({
    value: userSeats.length + i + 0,
    label: `${userSeats.length + i + 0} seats`,
  }));

  return (
    <>
      <div className={styles.logoutButton}>
        <AiOutlineLogout onClick={onLogout} />
      </div>
      <div className={styles.container}>
        <h2 className={styles.text}>Choose Seats</h2>
        <Screen />
        <div className={styles.seatcontainer}>
          <SeatLayout
            selectedSeats={selectedSeats}
            reservedSeats={reservedSeats}
            limit={seatCount}
            onSelectionChange={setSelectedSeats}
          />
        </div>
        <div className={styles.button}>
          <Button
            variant="secondary"
            label="Confirm"
            size="md"
            onClick={confirmHandler}
            disabled={selectedSeats.length !== seatCount}
          />
        </div>
        <div className={styles.legend}>
          <Legend />
        </div>
      </div>

      {showSelect && (
        <Modal
          title={
            userSeats.length > 0
              ? `You have already booked ${userSeats.length} seat(s). Add more?`
              : "Choose Number of Seats"
          }
          availableSeats={availableSeats}
          seatCount={seatCount}
          setSeatCount={setSeatCount}
          onClose={handleModal}
        >
          <SeatSelectForm
            seatOptions={seatOptions}
            seatCount={seatCount}
            setSeatCount={setSeatCount}
            onClose={handleModal}
          />
        </Modal>
      )}

      <ToastContainer position="top-right" autoClose={3000} />
    </>
  );
};
